@@ -1,7 +1,7 @@
 #' Speedy Word Embedding Association Test
 #'
-#' This functions test the bias in a set of word embeddings using the method by Caliskan et al (2017).
-#' @param w a numeric matrix of word embeddings (e.g. from rsparse::GloVe)
+#' This functions test the bias in a set of word embeddings using the method by Caliskan et al (2017). If possible, please use [query()] instead.
+#' @param w a numeric matrix of word embeddings (e.g. from [rsparse::GloVe()])
 #' @param S_words a character vector of the first set of target words. In an example of studying gender stereotype, it can include occupations such as programmer, engineer, scientists...
 #' @param T_words a character vector of the second set of target words. In an example of studying gender stereotype, it can include occupations such as nurse, teacher, librarian...
 #' @param A_words a character vector of the first set of attribute words. In an example of studying gender stereotype, it can include words such as man, male, he, his.
@@ -29,7 +29,7 @@
 #' weat_es(sw)
 #' @author Chung-hong Chan
 #' @references
-#' Caliskan, A., Bryson, J. J., & Narayanan, A. (2017). Semantics derived automatically from language corpora contain human-like biases. Science, 356(6334), 183-186.
+#' Caliskan, A., Bryson, J. J., & Narayanan, A. (2017). Semantics derived automatically from language corpora contain human-like biases. Science, 356(6334), 183-186. \doi{10.1126/science.aal4230}
 #' @export 
 weat <- function(w, S_words, T_words, A_words, B_words, verbose = FALSE) {
     w_lab <- rownames(w)
@@ -37,8 +37,10 @@ weat <- function(w, S_words, T_words, A_words, B_words, verbose = FALSE) {
     B_cleaned <- .clean(B_words, w_lab, verbose = verbose)
     S_cleaned <- .clean(S_words, w_lab, verbose = verbose)
     S_diff <- cpp_bweat(S_cleaned, A_cleaned, B_cleaned, w)
+    names(S_diff) <- S_cleaned
     T_cleaned <- .clean(T_words, w_lab, verbose = verbose)
     T_diff <- cpp_bweat(T_cleaned, A_cleaned, B_cleaned, w)
+    names(T_diff) <- T_cleaned
     res <- list(S_diff = S_diff, T_diff = T_diff, S_words = S_cleaned, T_words = T_cleaned, A_words = A_cleaned, B_words = B_cleaned)
     class(res) <- append(c("sweater", "weat"), class(res))
     return(res)
@@ -46,14 +48,14 @@ weat <- function(w, S_words, T_words, A_words, B_words, verbose = FALSE) {
 
 #' Calculation of WEAT effect size
 #'
-#' This function calculates the effect size from a sweater object. The original implementation in Caliskan et al. (2017) assumes the numbers of words in S and in T must be equal. The current implementation eases this assumption by adjusting the variance with the difference in sample sizes. It is also possible to convert the Cohen's d to Pearson's correlation coefficient (r).
+#' This function calculates the effect size from a sweater object. The original implementation in Caliskan et al. (2017) assumes the numbers of words in S and in T must be equal. The current implementation eases this assumption by adjusting the variance with the difference in sample sizes. It is also possible to convert the Cohen's d to Pearson's correlation coefficient (r). If possible, please use [calculate_es()] instead.
 #' @param x an object from the \link{weat} function.
 #' @param standardize a boolean to denote whether to correct the difference by the standard division. The standardized version can be interpreted the same way as Cohen's d.
 #' @param r a boolean to denote whether convert the effect size to biserial correlation coefficient.
 #' @return the effect size of the query
 #' @author Chung-hong Chan
 #' @references
-#' Caliskan, A., Bryson, J. J., & Narayanan, A. (2017). Semantics derived automatically from language corpora contain human-like biases. Science, 356(6334), 183-186.
+#' Caliskan, A., Bryson, J. J., & Narayanan, A. (2017). Semantics derived automatically from language corpora contain human-like biases. Science, 356(6334), 183-186. \doi{10.1126/science.aal4230}
 #' @examples
 #' # Reproduce the number in Caliskan et al. (2017) - Table 1, "Math vs. Arts"
 #' data(glove_math)
@@ -92,10 +94,13 @@ weat_es <- function(x, standardize = TRUE, r = FALSE) {
 }
 
 .exact_test <- function(S_diff, T_diff) {
-    s_length <- length(S_diff)
     union_diff <- c(S_diff, T_diff)
+    labels <- c(rep(TRUE, length(S_diff)), rep(FALSE, length(T_diff)))
     test_stat <- (mean(S_diff) - mean(T_diff))
-    return(cpp_exact(union_diff, test_stat, s_length))
+    permutations <- combinat::permn(union_diff)
+    st_diff <- purrr::map_dbl(permutations, ~ mean(.[labels]) - mean(.[!labels]))
+    p <- sum(st_diff > test_stat) / length(permutations)
+    return(p)
 }
 
 #' @rdname weat_resampling
@@ -106,7 +111,7 @@ weat_exact <- function(x) {
     if (length(c(S_diff, T_diff)) > 10) {
         warning("Exact test would take a long time. Use sweater_resampling or sweater_boot (to be implemented) instead.")
     }
-    p_value <- .exact_test(S_diff, T_diff)
+    p_value <- .exact_test(S_diff, T_diff)    
     res <- list(null.value = NULL, alternative = "greater", method = "The exact test in Caliskan et al. (2017)", estimate = NULL, data.name = deparse(substitute(x)), statistic = NULL, p.value = p_value)
     class(res) <- "htest"
     return(res)
@@ -120,7 +125,7 @@ weat_exact <- function(x) {
 #' @return A list with class \code{"htest"}
 #' @author Chung-hong Chan
 #' @references
-#' Caliskan, A., Bryson, J. J., & Narayanan, A. (2017). Semantics derived automatically from language corpora contain human-like biases. Science, 356(6334), 183-186.
+#' Caliskan, A., Bryson, J. J., & Narayanan, A. (2017). Semantics derived automatically from language corpora contain human-like biases. Science, 356(6334), 183-186. \doi{10.1126/science.aal4230}
 #' @examples
 #' # Reproduce the number in Caliskan et al. (2017) - Table 1, "Math vs. Arts"
 #' data(glove_math)
@@ -151,8 +156,6 @@ weat_resampling <- function(x, n_resampling = 9999) {
     p <- n_alter / n_resampling
     null_value <- mean(st_diff)
     attr(null_value, "names") <- "bias"
-    para <- null_value
-    attr(null_value, "names") <- "bias"
     res <- list(null.value = null_value, alternative = "greater", method = "Resampling approximation of the exact test in Caliskan et al. (2017)", estimate = test_stat, data.name = deparse(substitute(x)), statistic = test_stat, p.value = p)
     class(res) <- "htest"
     return(res)
@@ -162,6 +165,6 @@ weat_resampling <- function(x, n_resampling = 9999) {
 #'
 #' This is a subset of the original pretrained GLoVE word vectors provided by Pennington et al (2017). The same word vectors were used in Caliskan et al. (2017) to study biases.
 #' @references
-#' Pennington, J., Socher, R., & Manning, C. D. (2014, October). Glove: Global vectors for word representation. In Proceedings of the 2014 conference on empirical methods in natural language processing (EMNLP) (pp. 1532-1543).
-#' Caliskan, A., Bryson, J. J., & Narayanan, A. (2017). Semantics derived automatically from language corpora contain human-like biases. Science, 356(6334), 183-186.
+#' Pennington, J., Socher, R., & Manning, C. D. (2014, October). [Glove: Global vectors for word representation.](https://aclanthology.org/D14-1162/) In Proceedings of the 2014 conference on empirical methods in natural language processing (EMNLP) (pp. 1532-1543).
+#' Caliskan, A., Bryson, J. J., & Narayanan, A. (2017). Semantics derived automatically from language corpora contain human-like biases. Science, 356(6334), 183-186. \doi{10.1126/science.aal4230}
 "glove_math"
